//Imports libraries and initiate them
#include "HX711.h"
#include <LiquidCrystal_I2C.h>
#define PI 3.1415926535897932384626433832795
HX711 scale;
LiquidCrystal_I2C lcd(0x27, 16, 2);

// Define variables, changes these to your values:
#define LOADCELL_DOUT_PIN  9
#define LOADCELL_SCK_PIN  10
const int buttonPinMeasure = 13;
const int buttonPinTare = 7;
float calibration_factor = -213500.00;
const float length_cylinder = 0.75; // in meters
const float weight_cylinder_per_length = 0.78;// in kilograms
const float weight_end_cap = 0.013; // in kilograms
const float diameterTube = 75; // in milimeters
const float rho_w = 997; // in kg/m3
const float h_l = 0.23; //distance between bottom of tube to bottom anchor
const int delay_between_measurements = 1000; //delay between measuring in ms, change this to take faster measurements

// Calculate mass of the cylinder
const float massFloater = length_cylinder * weight_cylinder_per_length + weight_end_cap; // in kilograms

//Initiate the measurement device
void setup() {
  Serial.begin(9600);

  Serial.println("Arduino running, press button to make measurement...");
  lcd.init(); 

  scale.begin(LOADCELL_DOUT_PIN, LOADCELL_SCK_PIN);
  scale.tare();
  scale.set_scale(calibration_factor);

  lcd.backlight();
  lcd.setCursor(2,0);
  lcd.print("Water depth:");

  lcd.setCursor(0,1);
  lcd.print("--------------------------");
}

//Listen for button press
void loop() {

  //If 't' is sent through the serial monitor, the scale is tared
  if (Serial.available()) {
    char tare = Serial.read();
    if(tare == 't'){
      scale.tare();
      Serial.println("Taring the scale...");
    }
  }

  //Taring can also be done by pressing the dedicated tare button
  if (digitalRead(buttonPinTare) == HIGH) {
    Serial.println("Taring the scale...");
    scale.tare();
    delay(1000);
  }

  //If measuring button is pressed
  if (digitalRead(buttonPinMeasure) == HIGH) {
    
    //Calculate floating force and water levels  
    float F_l = -1 * scale.get_units() * 9.81;  
    float h_c = (4*massFloater*9.81 + 4*F_l) / (rho_w*9.81*3.14*pow(diameterTube/1000, 2));
    float h_w = h_c + h_l;

    //Determine sensor measuring range
    float h_w_min = (4*massFloater*9.81 + 4*0) / (rho_w*9.81*3.14*pow(diameterTube/1000, 2)) + h_l;
    float h_w_max = length_cylinder + h_l;

    // Serial.println(h_w); //For debugging

    //Dependent on the measured water level, show a message (either that the water depth is too low or too high
    //for what the sensor can measure, or the actual water depth if it lies in the measuring range)
    if (h_w <= h_w_min) {
      //Print to the serial monitor
      Serial.print("Water depth too low: minimum is ");
      Serial.print(h_w_min);
      Serial.println(" m");

      //Print to the LCD display
      lcd.setCursor(0,0);
      lcd.print("Water depth too");

      lcd.setCursor(0,1);
      lcd.print("                                        ");
      lcd.setCursor(0,1);
      String printLine = String("low, min ") + h_w_min + String(" m");
      lcd.print(printLine);

      delay(delay_between_measurements);
    } else if (h_w >= h_w_max) {
      //Print to the serial monitor
      Serial.print("Water depth too high: maximum is ");
      Serial.print(h_w_max);
      Serial.println(" m");

      //Print to the LCD display
      lcd.setCursor(0,0);
      lcd.print("Water depth too");

      lcd.setCursor(0,1);
      lcd.print("                                        ");
      lcd.setCursor(0,1);
      String printLine = String("high, max ") + h_w_max + String(" m");
      lcd.print(printLine);

      delay(delay_between_measurements);
    } else {
      //Print to the serial monitor
      Serial.print("Water depth: ");
      Serial.print(h_w);
      Serial.println(" m");

      //Print to the LCD display
      lcd.setCursor(0,0);
      lcd.print("                                        ");
      lcd.setCursor(2,0);
      lcd.print("Water depth:");

      lcd.setCursor(0,1);
      lcd.print("                                        ");
      lcd.setCursor(5,1);
      String printLine = String(h_w) + " m";
      lcd.print(printLine);

      delay(delay_between_measurements);
    } 
  }
}
